#include "manymapper.h"

#include <QWidget>
#include <QAbstractItemModel>

ManyMapper::ManyMapper(QObject *parent) : QDataWidgetMapper(parent)
{
}

void ManyMapper::setModel(QAbstractItemModel *model) {
  QDataWidgetMapper::setModel(model);
  connect(model, &QAbstractItemModel::dataChanged, this, [this](){
    this->setCurrentIndex(this->_currentIndex);
  });
}

void ManyMapper::widgetChanged() {
  auto model = this->model();
  auto sender = this->sender();

  for (Mapping &mapping : _widgetList) {
    if (mapping.widget == nullptr || mapping.widget != sender) continue;
    bool wasBlocked = mapping.widget->blockSignals(true);
    auto mindex = model->index(_currentIndex, mapping.section);
    model->setData(mindex, mapping.prop.read(mapping.widget));
    mapping.widget->blockSignals(wasBlocked);
  }
}

void ManyMapper::addMapping(QWidget *widget, int section, QByteArray propertyName) {
  // we use the widget's meta object in order to get us a signal that we can listen to
  // so we know when this property changes
  auto widgetMetaObject = widget->metaObject();
  auto property = widgetMetaObject->userProperty();
  if (!propertyName.isEmpty())
    property = widgetMetaObject->property(widgetMetaObject->indexOfProperty(propertyName));
  auto metaObject = this->metaObject();
  QMetaObject::connect(widget, property.notifySignalIndex(), this, metaObject->indexOfSlot("widgetChanged()"));
  Mapping mapping = {widget,section,property};
  _widgetList.append(mapping);
}

void ManyMapper::setCurrentIndex(int index) {
  _currentIndex = index;
  // when the current index changes all the widgets have to be reloaded from the model
  // we do not want them to signal property changes to us while they are reloading
  // so we have to block their signals until the current index is changed and then
  // unblock them afterwards if they were not blocked when this method entered
  // NOTE: this probably is not threadsafe or reentrant, just like the rest of Qt
  // and any other GUI framework, so take care when synchronizing it with other threads
  for (Mapping &mapping : _widgetList) {
    if (mapping.widget == nullptr) continue;
    bool wasBlocked = mapping.widget->blockSignals(true);
    auto mindex = this->model()->index(index, mapping.section);
    mapping.widget->setProperty(mapping.prop.name(), mindex.data());
    mapping.widget->blockSignals(wasBlocked);
  }
}
